from sage.schemes.hyperelliptic_curves.hypellfrob import hypellfrob
from sage.schemes.hyperelliptic_curves.monsky_washnitzer import matrix_of_frobenius_hyperelliptic

p=7
T.<x>=PolynomialRing(ZZ);
S.<x0,x1,x2>=PolynomialRing(GF(p^2));

f=x^7+x+1

#Is y^2=f isomorphic to y^2=x^8-1 over F7?
(x-1)^8*f(1/(x-1))

#Compute matrix of Frobenius
A, dum=matrix_of_frobenius_hyperelliptic(f,7,3)

A=A.delete_columns([0,1,2]);
A=A.delete_rows([3,4,5]);
A=A.change_ring(GF(p));
A=A.change_ring(S);

#Compute the polynomials from Köck & Tait "On the de-Rham cohomology of hyperelliptic curves"
s1=x*f.derivative()-2*f;
s2=x*f.derivative()-2*2*f;
s3=x*f.derivative()-2*3*f;


phi1=s1 // x^2;
phi2=s2 // x^3;
phi3=s3 // x^4;


phi1, phi2, phi3;

Fer=matrix([[3*x2,5*x1,x0]])*A*matrix([[x0^p],[x1^p], [x2^p]]);
Fer=Fer[0,0];
#Equation of the Fermat curve in the coordinate system given by C
Fer;

#Compute the Fp2-rational intersection points
# of the Fermat curve with the image of the canonical map
I=ideal([Fer, x2*x0-x1^2, x0-1]);
var = I.variety()

Q=x0*x2-x1^2
for va in var:
      P=[1, va["x1"], va["x2"]];
      print(Q.derivative(x0)(P)*x0+Q.derivative(x1)(P)*x1+Q.derivative(x2)(P)*x2);
      print(Fer.derivative(x0)(P)*x0+Fer.derivative(x1)(P)*x1+Fer.derivative(x2)(P)*x2);
      print((Fer.derivative(x0)(P)*x0+Fer.derivative(x1)(P)*x1+Fer.derivative(x2)(P)*x2) % (Q.derivative(x0)(P)*x0+Q.derivative(x1)(P)*x1+Q.derivative(x2)(P)*x2));
      print("\n");

#The desired points are the points of tangential intersection.
#We find exactly one. Therefore there are p^2=49 components of the formal completion for which [C] is a point of non transversal intersection.

#Some sanity checks for Fer:
g=21;
P=ProjectiveSpace(S);
C=Curve(Fer, P);
C=C.change_ring(GF(p^2));
print(C.arithmetic_genus()==g);
points=C.rational_points();
print(len(points)==p^2+2*p*g+1);
